#!/usr/bin/env python
import builtins
import inspect
from getclass import getclass
from objectname import objectname
from isclassmethod import isclassmethod
from islambda import islambda
from isstaticmethod import isstaticmethod
from isobject import isobject
from public import public


def isproperty(obj):
    return isinstance(obj, property)


def gettype(obj):
    if inspect.ismodule(obj):
        return "module"
    if inspect.isclass(obj):
        return "class"
    if inspect.ismethod(obj):
        return"method"
    if inspect.isfunction(obj):
        return "function"
    if islambda(obj):
        return"lambda function"
    if isclassmethod(obj):
        return"classmethod"
    if isstaticmethod(obj):
        return"staticmethod"
    if isproperty(obj):
        return "property"
    if inspect.isbuiltin(obj) and inspect.isroutine(obj):
        return"builtin function"


def getattr_info(obj, name):
    # no default :) but we can show more info for debug
    fullname = objectname(obj, fullname=True)
    _type = gettype(obj)
    err = None
    # todo: isgenerator, istraceback, etc...

    if inspect.isbuiltin(obj) and inspect.isroutine(obj):
        _type = "builtin function"
        fullname = fullname.split(".")[-1]
    if isobject(obj) and getclass(obj) is not None:
        # default: 'clsname' object has no attribute 'name'
        cls = getclass(obj)
        if cls:
            fullname = objectname(cls, fullname=True)
            err = "%s object has no attribute '%s'" % (fullname, name)
            # _type = "object"
    if _type:
        err = "%s '%s' has no attribute '%s'" % (_type, fullname, name)
    if err:
        raise AttributeError(err)
    # default
    builtins.getattr(obj, name)
    # raise AttributeError(str(e))


@public
def getattr(obj, name, default=None):
    if hasattr(obj, name):  # exists
        return builtins.getattr(obj, name)
    # attr not exists
    if default is not None:
        return default  # default
    return getattr_info(obj, name)
